package PureMessage::Service::logsearch;
use strict;
use warnings;
use base 'PureMessage::Service';
use Carp qw(croak);
use ActiveState::Handy qw(run);
use PureMessage::Config qw(expand_filename expand_config_filename);
use ActiveState::File::Atomic;

my $logsearch_d = expand_filename("etc/init.d/pmx-logsearch-index");

sub Available {
    my $class = shift;
    return ($class->new(name => "Log Search Index"));
}

sub start {
    my ($self, %opts) = @_;
    _exec([$logsearch_d, "start"], %opts);
}

sub stop {
    my ($self, %opts) = @_;
    _exec([$logsearch_d, "stop"], %opts);
}

sub restart {
    my ($self, %opts) = @_;
    _exec([$logsearch_d, "restart"], %opts);
}

sub status {
    my %s = (running => 0);
    
    open(my $fh, "$logsearch_d status |")
        or die "Failed to exec '$logsearch_d': $!";
    my $contents = do { local $/; <$fh>; };
    chomp $contents;
    my $ok = close $fh || ($? >> 8) == 2;

    unless ($ok) {
        $s{ErrorMsg} = "Error retrieving status from Logsearch: exited with $?";
    }

    my ($pid) = $contents =~ m{pid\s*:\s*(\d+)}msi;

    if ($pid) {
        $s{pid} = $pid;
        $s{running} = 1;
        $s{ready} = 1;
    }
    return \%s;
}

sub running {
    return status()->{running};
}

sub _exec {
    my $cmd = shift;
    my %opts = @_;

    if ($opts{bg}) {
        my $pid = fork;
        die "Can't fork: $!" unless defined $pid;
        return $pid if $pid;
        open(STDIN, "</dev/null");
        open(STDOUT, ">/dev/null");
        open(STDERR, ">/dev/null");
        exec(@$cmd);
        die "Failed to exec @$cmd: $!";
    }
    run('@-' . join(' ', @$cmd) . ' >/dev/null 2>&1');
}

1;
